{Torry's Button v1.0 (Delphi 1.x and 2.x)
 Copyright  Maxim Peresada, June 1997

 Base on TOfficeButton by Alexander Meeder and TLightCheck by Frederic Vanmol

Torry's Button is transparent speedbutton that looks like common label until you don't
move mouse cursor over it. Also it has "Led-like" indicator that has different colors for 
different button's states.


 Maxim Peresada
 "Torry's Delphi Pages"
 http://carbohyd.siobc.ras.ru/torry/
 e-mail: mperesada@glas.apc.org, torry@ibch.siobc.ras.ru

 Alexsander Meeder
 http://huizen.dds.nl/~ameeder
 e-mail: ameeder@dds.nl

}

unit Torrybtn;

interface

uses
  {$IFDEF WIN32}
    Windows,
  {$ELSE}
    WinProcs, WinTypes,
  {$ENDIF}
    SysUtils, Messages, Classes, Graphics, Controls;

type
  TTorryButton = class(TGraphicControl)
  private
    FBorderColor: TColor;
    FShadowColor: TColor;
    FLedOffColor: TColor;
    FLedOnColor: TColor;
    FLedDownColor: TColor;
    FLedPosition: Integer;
    FSpace : Integer;
    FCaption: TCaption;
    FEnabled: boolean;
    FFont: TFont;
    FRoundSize: integer;

    FMouseEnter: TNotifyEvent;
    FMouseLeave: TNotifyEvent;

    HasMouse: boolean;
    IsMouseDown: boolean;
    ExtraSpace: word;
    procedure SetBorderColor(value: TColor);
    procedure SetShadowColor(value: TColor);
    procedure SetLedOffColor(value: TColor);
    procedure SetLedOnColor(value: TColor);
    procedure SetLedDownColor(value: TColor);
    procedure SetLedPosition(value: Integer);
    procedure SetSpace(value: Integer);
    procedure SetCaption(value: TCaption);
    procedure SetEnabled(value: boolean);
    procedure SetFont(value: TFont);
    procedure SetRoundSize(value: integer);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure WMLButtonDblClick(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMMButtonDblClick(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMRButtonDblClick(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BorderColor: TColor read FBorderColor write SetBorderColor default clWhite;
    property ShadowColor: TColor read FShadowColor write SetShadowColor default clGray;
    property LedOffColor: TColor read FLedOffColor write SetLedOffColor default clGray;
    property LedOnColor: TColor read FLedOnColor write SetLedOnColor default clNavy;
    property LedDownColor: TColor read FLedDownColor write SetLedDownColor default clLime;
    property LedPosition: Integer read FLedPosition write SetLedPosition default 10;
    property Space : Integer read FSpace write SetSpace default 10;
    property Caption: TCaption read FCaption write SetCaption;
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Font: TFont read FFont write SetFont;
    property RoundSize: integer read FRoundSize write SetRoundSize default 20;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TNotifyEvent read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;


procedure Register;

implementation

{$IFDEF WIN32}
{$R *.D32}
{$ELSE}
{$R *.D16}
{$ENDIF}


const
  BorderWidth = 1;
  LedSize     = 7;

procedure TTorryButton.SetBorderColor(value: TColor);
begin
  if Value <> FBorderColor then
  begin
    FBorderColor := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetShadowColor(value: TColor);
begin
  if Value <> FShadowColor then
  begin
    FShadowColor := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetLedOffColor(value: TColor);
begin
  if Value <> FLedOffColor then
  begin
    FLedOffColor := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetLedOnColor(value: TColor);
begin
  if Value <> FLedOnColor then
  begin
    FLedOnColor := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetLedDownColor(value: TColor);
begin
  if Value <> FLedDownColor then
  begin
    FLedDownColor := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetLedPosition(value: Integer);
begin
  if Value <> FLedPosition then
  begin
    FLedPosition := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetSpace(value: Integer);
begin
  if Value <> FSpace then
  begin
    FSpace := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetCaption(value: TCaption);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Repaint;
  end;
end;

procedure TTorryButton.SetEnabled(value: boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    HasMouse := false;
    IsMouseDown := false;
    ExtraSpace := 0;
    Repaint;
  end;
end;

procedure TTorryButton.SetFont(value: TFont);
begin
  if Value <> FFont then
  begin
    FFont.Assign(Value);
    Repaint;
  end;
end;

procedure TTorryButton.SetRoundSize(value: integer);
begin
  if Value <> FRoundSize then
  begin
    FRoundSize := Value;
    Repaint;
  end;
end;

procedure TTorryButton.CMMouseEnter(var Message: TMessage);
begin
  if Enabled then
  begin
    HasMouse := true;
    if IsMouseDown then ExtraSpace := 1 else ExtraSpace := 0;
    Repaint;
    if Assigned(FMouseEnter) then FMouseEnter(Self);
  end;
end;

procedure TTorryButton.CMMouseLeave(var Message: TMessage);
begin
  if Enabled then
  begin
    HasMouse := false;
    ExtraSpace := 0;
    Repaint;
    if Assigned(FMouseLeave) then FMouseLeave(Self);
  end;
end;

procedure TTorryButton.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  if Enabled then
  begin
    CurrentPoint.X := Message.XPos;
    CurrentPoint.Y := Message.YPos;
    if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
    begin
      HasMouse := true;
      Repaint;
    end;
  end;
  inherited;
end;

procedure TTorryButton.WMLButtonDblClick(var Message: TWMLButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TTorryButton.WMMButtonDblClick(var Message: TWMMButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TTorryButton.WMRButtonDblClick(var Message: TWMRButtonDown);
begin
  IsMouseDown := false;
  if Enabled then inherited;
end;

procedure TTorryButton.Click;
begin
  if Enabled then inherited Click;
end;

procedure TTorryButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FEnabled then
  begin
    HasMouse := true;
    IsMouseDown := true;
    ExtraSpace := 1;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TTorryButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FEnabled then
  begin
    IsMouseDown := false;
    ExtraSpace := 0;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TTorryButton.Paint;
var
 LedTop, LedLeft, ESpace: Integer;
begin
  LockWindowUpdate(Canvas.Handle);
  inherited Paint;
  if HasMouse then begin
   LedTop := ((Height - LedSize) div 2) + ExtraSpace-1;
   LedLeft := LedPosition + ExtraSpace-1;
  end
  else begin
   LedTop := ((Height - LedSize) div 2) + ExtraSpace;
   LedLeft := LedPosition + ExtraSpace;
  end;
  with Canvas do begin
    Brush.Style := bsClear;
    Brush.Color := Color;
    Font.Assign(FFont);
    Pen.Width := BorderWidth;
    if HasMouse then begin
     {Down}
     if IsMouseDown then begin
      {Remove old frame}
      Pen.Color := Color;
      RoundRect(0,0,Width-1,Height-1,FRoundSize,FRoundSize);
      {DrawFrame}
      Pen.Color := BorderColor;
      RoundRect(1,1,Width,Height,FRoundSize,FRoundSize);
      {Draw Button}
      Pen.Color := Brush.Color;
      RoundRect(2,2,Width-1,Height-1,FRoundSize,FRoundSize);
     end
     {On}
     else begin
      {DrawShadow}
      Pen.Color := ShadowColor;
      RoundRect(1,1,Width,Height,FRoundSize,FRoundSize);
      {DrawFrame}
      Pen.Color := BorderColor;
      RoundRect(0,0,Width-1,Height-1,FRoundSize,FRoundSize);
      {Draw Button}
      Pen.Color := Brush.Color;
      RoundRect(1,1,Width-2,Height-2,FRoundSize,FRoundSize);
     end;
    end
 {Off}
    else begin
     {Remove old frame}
     Pen.Color := Color;
     RoundRect(0,0,Width-1,Height-1,FRoundSize,FRoundSize);
     {Remove shadow}
     RoundRect(1,1,Width,Height,FRoundSize,FRoundSize);
     {Draw Button}
     RoundRect(2,2,Width-1,Height-1,FRoundSize,FRoundSize);
    end;
    if HasMouse then
    begin
       if IsMouseDown then
          Pen.Color := FLedDownColor
       else
          Pen.Color := FLedOnColor;
    end
    else Pen.Color := FLedOffColor;
    {  7x7 }
    Pixels[LedLeft+2, LedTop] := clBtnShadow;
    Pixels[LedLeft+3, LedTop] := clBtnShadow;
    Pixels[LedLeft+4, LedTop] := clBtnShadow;
    Pixels[LedLeft+1, LedTop+1] := clBtnShadow;
    Pixels[LedLeft, LedTop+2] := clBtnShadow;
    Pixels[LedLeft, LedTop+3] := clBtnShadow;
    Pixels[LedLeft, LedTop+4] := clBtnShadow;
    Pixels[LedLeft+1, LedTop+5] := clBtnShadow;

    Pixels[LedLeft+6, LedTop+2] := clBtnHighLight;
    Pixels[LedLeft+6, LedTop+3] := clBtnHighLight;
    Pixels[LedLeft+6, LedTop+4] := clBtnHighLight;
    Pixels[LedLeft+5, LedTop+1] := clBtnHighLight;
    Pixels[LedLeft+2, LedTop+6] := clBtnHighLight;
    Pixels[LedLeft+3, LedTop+6] := clBtnHighLight;
    Pixels[LedLeft+4, LedTop+6] := clBtnHighLight;
    Pixels[LedLeft+5, LedTop+5] := clBtnHighLight;
    Pixels[LedLeft+2, LedTop+3] := clBtnHighLight;

    Pixels[LedLeft+2, LedTop+1] := Pen.Color;
    Pixels[LedLeft+3, LedTop+1] := Pen.Color;
    Pixels[LedLeft+4, LedTop+1] := Pen.Color;
    Pixels[LedLeft+1, LedTop+2] := Pen.Color;
    Pixels[LedLeft+2, LedTop+2] := Pen.Color;
    Pixels[LedLeft+3, LedTop+2] := Pen.Color;
    Pixels[LedLeft+4, LedTop+2] := Pen.Color;
    Pixels[LedLeft+5, LedTop+2] := Pen.Color;
    Pixels[LedLeft+1, LedTop+3] := Pen.Color;
    Pixels[LedLeft+3, LedTop+3] := Pen.Color;
    Pixels[LedLeft+4, LedTop+3] := Pen.Color;
    Pixels[LedLeft+5, LedTop+3] := Pen.Color;
    Pixels[LedLeft+1, LedTop+4] := Pen.Color;
    Pixels[LedLeft+2, LedTop+4] := Pen.Color;
    Pixels[LedLeft+3, LedTop+4] := Pen.Color;
    Pixels[LedLeft+4, LedTop+4] := Pen.Color;
    Pixels[LedLeft+5, LedTop+4] := Pen.Color;
    Pixels[LedLeft+2, LedTop+5] := Pen.Color;
    Pixels[LedLeft+3, LedTop+5] := Pen.Color;
    Pixels[LedLeft+4, LedTop+5] := Pen.Color;
{  7x7 }

    if not FEnabled then Font.Color := clGrayText;
    if HasMouse then begin
     if IsMouseDown then ESpace:=1 else ESpace:=0;
    end
    else ESpace:=1;
  TextOut(((Width-TextWidth(FCaption)) div 2)+Space+ESpace,
             ((Height - TextHeight(FCaption)) div 2)+ESpace,FCaption);
  end;
  LockWindowUpdate(0);
end;

constructor TTorryButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
  FFont := TFont.Create;
  {$IFNDEF WIN32}
    FFont.Name := 'MS Sans Serif';
    FFont.Size := 8;
    FFont.Style := [];
  {$ENDIF}
  FBorderColor := clWhite;
  FShadowColor := clGray;
  FLedOffColor := clSilver;
  FLedOnColor := clNavy;
  FLedDownColor := clLime;
  FCaption := 'Torry Button';
  FEnabled := true;
  FRoundSize := 20;
  FLedPosition:= FRoundSize div 2;
  FSpace:=FLedPosition;
  HasMouse := false;
  IsMouseDown := false;
  ExtraSpace := 0;
  Width := 100;
  Height := 25;
end;

destructor TTorryButton.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;


procedure Register;
begin
  RegisterComponents('Torry', [TTorryButton]);
end;

end.
